#!/usr/bin/env python3

import json
import logging
import pathlib
import queue
import threading

from . import config
from . import controller
from . import dect

logger = logging.getLogger("fieldpoc.fieldpoc")

class FieldPOC:
    config = None
    extensions = None

    def __init__(self, config_file_path, extensions_file_path):
        logger.info("loading configuration")
        self.config_file_path = pathlib.Path(config_file_path)
        self._load_config()
        self.extensions_file_path = pathlib.Path(extensions_file_path)
        self._load_extensions()

        logger.info("initialising queues")
        self.queues = {
            "controller": queue.Queue(),
            "dect": queue.Queue(),
        }

        logger.info("initialising components")
        self._controller = controller.Controller(self)
        self._dect = dect.Dect(self)

    def queue_all(self, msg):
        """
            Send message to every queue
        """

        for queue in self.queues.values():
            queue.put(msg)

    def run(self):
        logger.info("stating components")

        self._controller_thread = threading.Thread(target=self._controller.run)
        self._controller_thread.start()

        self._dect_thread = threading.Thread(target=self._dect.run)
        self._dect_thread.start()

        logger.info("started components")

    def _load_config(self):
        self.config = config.Config(json.loads(self.config_file_path.read_text()))

    def _load_extensions(self):
        self.extensions = config.Extensions(json.loads(self.extensions_file_path.read_text()))
